#!/usr/bin/env node
/*!
 * gitmars v5.1.1-beta.1
 * 这是一个git工作流工具
 * (c) 2021-2023 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { program } from 'commander';
import dayjs from 'dayjs';
import inquirer from 'inquirer';
import sh from 'shelljs';
import chalk from 'chalk';
import { queue } from '@gitmars/core/lib/queue';
import getIsGitProject from '@gitmars/core/lib/git/getIsGitProject';
import getCurrentBranch from '@gitmars/core/lib/git/getCurrentBranch';
import checkGitStatus from '@gitmars/core/lib/git/checkGitStatus';
import fetch from '@gitmars/core/lib/git/fetch';
import prune from '@gitmars/core/lib/git/prune';
import getGitLogs from '@gitmars/core/lib/git/getGitLogs';
import getGitLogsByCommitIDs from '@gitmars/core/lib/git/getGitLogsByCommitIDs';
import searchBranches from '@gitmars/core/lib/git/searchBranches';
import { createArgs } from '@gitmars/core/lib/utils/command';
import echo from '@gitmars/core/lib/utils/echo';
import copyConfig from '#lib/conf/copy';
import lang from '#lib/common/local';

const { t } = lang;
const { blue, green, red, yellow } = chalk;
const { args, options } = copyConfig;
if (!getIsGitProject()) {
  sh.echo(red(t("The current directory is not a git project directory")));
  process.exit(1);
}
program.name("gitm copy").usage("[commitid...] [--lastet [lastet]] [--limit [limit]] [--no-merges]").description(
  t(
    "cherry-pick batch version, copy a record from a branch and merge it into the current branch"
  )
);
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action(async (commitid, opts) => {
  const keys = ["%H", "%T", "%P", "%aI", "%an", "%s", "%b"];
  const current = getCurrentBranch();
  const status = checkGitStatus();
  let logList = [], cmd = [], commitIDs = [];
  if (!status)
    process.exit(1);
  fetch();
  prune();
  const branches = searchBranches();
  if (commitid.length > 0) {
    logList = getGitLogsByCommitIDs({ commitIDs: commitid, keys });
  } else {
    logList = getGitLogs({
      lastet: opts.lastet,
      limit: opts.limit,
      noMerges: !opts.merges,
      keys
    });
    if (logList.length === 0) {
      echo(
        yellow(
          t(
            "No eligible commit logs found, please relax the filtering conditions appropriately. The process has been exited"
          )
        )
      );
      process.exit(0);
    }
  }
  const prompt = [
    {
      type: "checkbox",
      message: t("Please select the commit record to copy"),
      name: "commitIDs",
      choices: []
    },
    {
      type: "list",
      message: t("Please select the target branch"),
      name: "branch",
      choices: branches,
      when(answers2) {
        return answers2.commitIDs.length;
      }
    }
  ];
  logList.forEach((log, index) => {
    const _time = dayjs(log["%aI"]).format("YYYY/MM/DD HH:mm");
    prompt[0].choices.push({
      name: `${green(index + 1 + ".")} ${green(log["%s"])} | ${yellow(log["%an"])} | ${blue(
        _time
      )}`,
      value: log["%H"],
      checked: false
    });
  });
  const answers = await inquirer.prompt(prompt);
  commitIDs = answers.commitIDs;
  if (commitIDs.length === 0) {
    echo(yellow(t("No commit record selected, process has exited")));
    process.exit(0);
  }
  cmd = [
    `git checkout ${answers.branch}`,
    "git pull",
    {
      cmd: `git cherry-pick ${commitIDs.reverse().join(" ")}`,
      config: {
        again: false,
        success: t("Record merge successful"),
        fail: t("Merge failed, please follow the instructions")
      }
    },
    `git checkout ${current}`
  ];
  queue(cmd);
});
program.parse(process.argv);
