#!/usr/bin/env node
/*!
 * gitmars v5.1.1-beta.1
 * 这是一个git工作流工具
 * (c) 2021-2023 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { program } from 'commander';
import dayjs from 'dayjs';
import inquirer from 'inquirer';
import sh from 'shelljs';
import chalk from 'chalk';
import { queue } from '@gitmars/core/lib/queue';
import getIsGitProject from '@gitmars/core/lib/git/getIsGitProject';
import getGitLogs from '@gitmars/core/lib/git/getGitLogs';
import getGitLogsByCommitIDs from '@gitmars/core/lib/git/getGitLogsByCommitIDs';
import getCurrentBranch from '@gitmars/core/lib/git/getCurrentBranch';
import { createArgs } from '@gitmars/core/lib/utils/command';
import echo from '@gitmars/core/lib/utils/echo';
import { addRevertCache, getRevertCache, setRevertCache } from '@gitmars/core/lib/cache/revertCache';
import lang from '#lib/common/local';
import undoConfig from '#lib/conf/undo';

const { t } = lang;
const { blue, green, red, yellow } = chalk;
const { args, options } = undoConfig;
if (!getIsGitProject()) {
  sh.echo(red(t("The current directory is not a git project directory")));
  process.exit(1);
}
function getRevertCommitIDs(commitIDs) {
  const revertCache = getRevertCache();
  const current = getCurrentBranch();
  let len = commitIDs.length;
  while (len--) {
    const _before = revertCache.findIndex(
      (item) => item.branch === current && item.before["%H"] === commitIDs[len]
    );
    const _after = revertCache.findIndex(
      (item) => item.branch === current && item.after["%H"] === commitIDs[len]
    );
    if (_before > -1 || _after > -1) {
      echo(
        yellow(
          _before > -1 ? t(
            "Detected {id} This record has been revert once, please check if there is an error",
            { id: commitIDs[len] }
          ) : t(
            'The record {id} is detected as a revert record, please use the "gitm redo" operation',
            { id: commitIDs[len] }
          )
        )
      );
      commitIDs.splice(len, 1);
    }
  }
  return commitIDs;
}
function calculate(all = false, opt) {
  const keys = ["%H", "%T", "%P", "%aI", "%an", "%s", "%b"];
  const revertCache = getRevertCache();
  const current = getCurrentBranch();
  let len = revertCache.length;
  while (len--) {
    const { before, after, branch } = revertCache[len];
    let _undoLogs = [], _redoLogs = [];
    if (!after) {
      const _logs = getGitLogs({
        lastet: opt.lastet,
        limit: opt.limit * 2,
        noMerges: true,
        keys,
        branch,
        grep: before["%H"]
      });
      if (_logs && _logs.length > 0)
        revertCache[len].after = _logs[0];
    }
    _undoLogs = getGitLogs({
      lastet: "60d",
      limit: 500,
      noMerges: true,
      keys,
      branch,
      grep: before["%H"]
    });
    if (after) {
      _redoLogs = getGitLogs({
        lastet: "60d",
        limit: 500,
        noMerges: true,
        keys,
        branch,
        grep: after["%H"]
      });
    }
    if (!all && current !== branch)
      continue;
    if (_undoLogs.length === 0 || _redoLogs.length > 0) {
      echo(
        yellow(
          _undoLogs.length === 0 ? t(
            "Detected that {id} has failed to undo this record and has deleted the related logs",
            { id: revertCache[len].before["%H"] }
          ) : t(
            "The record {id} was detected to have been recovered, and the related logs were deleted",
            { id: revertCache[len].before["%H"] }
          )
        )
      );
      revertCache.splice(len, 1);
    }
  }
  setRevertCache(revertCache);
}
program.name("gitm undo").usage("[commitid...] [--lastet [lastet]] [--limit [limit]] [-m --mode [mode]] [--no-merges]").description(t("Undo a commit record"));
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action(async (commitid, opt) => {
  const keys = ["%H", "%T", "%P", "%aI", "%an", "%s", "%b"];
  const current = getCurrentBranch();
  let logList = [], cmd = [], commitIDs = [], mode = "";
  if (opt.calc) {
    calculate(false, opt);
    return;
  }
  if (opt.calcAll) {
    calculate(true, opt);
    return;
  }
  if (!opt.limit)
    opt.limit = 20;
  mode = " -m " + Math.abs(Number(opt.mode || 1));
  if (commitid.length > 0) {
    logList = getGitLogsByCommitIDs({ commitIDs: commitid, keys });
  } else {
    logList = getGitLogs({
      lastet: opt.lastet,
      limit: opt.limit,
      noMerges: !opt.merges,
      keys
    });
    if (logList.length === 0) {
      echo(
        yellow(
          t(
            "No eligible commit logs found, please relax the filtering conditions appropriately. The process has been exited"
          )
        )
      );
      process.exit(0);
    } else {
      const prompt = {
        type: "checkbox",
        message: t("Please select the commit record to undo."),
        name: "commitIDs",
        choices: []
      };
      logList.forEach((log, index) => {
        const _time = dayjs(log["%aI"]).format("YYYY/MM/DD HH:mm");
        prompt.choices.push({
          name: `${green(index + 1 + ".")} ${green(log["%s"])} | ${yellow(
            log["%an"]
          )} | ${blue(_time)}`,
          value: log["%H"],
          checked: false
        });
      });
      commitIDs = (await inquirer.prompt(prompt)).commitIDs;
    }
  }
  if (commitIDs.length === 0) {
    echo(yellow(t("No commit record selected, process has exited")));
    process.exit(0);
  }
  commitIDs = getRevertCommitIDs(commitIDs);
  if (commitIDs.length === 0) {
    echo(yellow(t("There are no revocable records, the process has exited")));
    process.exit(0);
  }
  logList = logList.filter((log) => commitIDs.some((id) => log["%H"].includes(id)));
  cmd = logList.map((log) => {
    return {
      cmd: `git revert -s --no-edit ${log["%H"]}${mode}`,
      config: {
        again: false,
        success: t("Undo successfully: {something}", {
          something: log["%s"]
        }),
        fail: t("An error has occurred, please follow the instructions")
      }
    };
  });
  const revertCacheList = logList.map((log) => {
    const cache = {
      before: log,
      after: null,
      branch: current
    };
    const _logs = getGitLogs({
      lastet: opt.lastet,
      limit: opt.limit * 2,
      noMerges: true,
      keys,
      branch: current,
      grep: log["%H"]
    });
    if (_logs.length > 0)
      cache.after = _logs[0];
    return cache;
  });
  addRevertCache(revertCacheList);
  queue(cmd).then(() => {
    calculate(false, opt);
  }).catch(() => {
    echo(yellow(t("After handling conflicts, execute: gitm undo --calc")));
  });
});
program.parse(process.argv);
